#! /usr/bin/python
# -*- coding: UTF-8 -*-

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Wnck', '3.0')
# gi.require_version('Fp', '1.0')
from gi.repository import Gtk, Gdk, Pango, GdkPixbuf, GLib, Wnck, GObject
from fp_pam import FPData, Fingerprint as Fp
import gettext
import os
import signal
import fcntl
import sys
import dbus

# test
import time

gettext.bindtextdomain("cdos-desktop", "/usr/share/cdos-de/locale")
gettext.textdomain("cdos-desktop")
_ = gettext.gettext

fpDev = None
print fpDev

cssfile = "/usr/lib/cdos-settings-fingerprint/fingerprint.css"
imagename = "/usr/lib/cdos-settings-fingerprint/fingerprint.png"
# test
# filename = "/etc/fingerprint"

# filedata = [["billy", 1, "Fingerprint1"], ["billy", 2, "手指2"], ["billy", 3, "手指3"], ["test1", 4, "手指1"], ["test1", 5, "手指2"], ["test2", 6, "zdj"], ["root", 7, "手指1"], ["root", 8, "fin"]]

def get_current_login_user():
    # print "login user= %s" % os.getlogin()
    return os.getlogin()

class Lock:
    def __init__(self, filename):
        self.handle = open(filename, 'r')

    def acquire(self):
        fcntl.flock(self.handle, fcntl.LOCK_EX | fcntl.LOCK_NB)

    def release(self):
        fcntl.flock(self.handle, fcntl.LOCK_UN)

    def __del__(self):
        self.handle.close()

class FpDev(Fp):
    def __init__(self):
        # self.fp_dev = Fp()
        super(FpDev, self).__init__()
        self.dialog = None
        self.enroll_one_image_id = 0
        self.enroll_finished_id = 0
        self.catched_error_id = 0
        self.device_count = 0

        self.bus = dbus.SystemBus()
        self.proxy = self.bus.get_object('com.nfs.Fingerprint', '/com/nfs/FingerprintObject')
        self.iface = dbus.Interface(self.proxy, dbus_interface='com.nfs.fingerprint')

    def connect_signal(self, dialog):
        if not self.dialog:
            self.dialog = dialog
            # get fingerprint feature success send "enroll-one-image" signal
            self.enroll_one_image_id = self.connect("enroll-one-image", dialog.on_enroll_one_image)
            self.enroll_finished_id = self.connect("enroll-finished", dialog.on_enroll_finished)
            self.catched_error_id = self.connect("catched-error", dialog.on_catched_error)
            self.catched_canceled_id = self.connect("catched-canceled", dialog.on_catched_canceled)

    def disconnect_signal(self, dialog):
        self.dialog = dialog
        if self.enroll_one_image_id > 0:
            self.handler_disconnect(self.enroll_one_image_id)
        if self.enroll_finished_id > 0:
            self.handler_disconnect(self.enroll_finished_id)
        if self.catched_error_id > 0:
            self.handler_disconnect(self.catched_error_id)
        if self.catched_canceled_id > 0:
            self.handler_disconnect(self.catched_canceled_id)

    def get_max_fingerprint(self):
        return 3

    def start_enroll(self):
        print "in ", sys._getframe().f_code.co_name  # debug
        return self.enroll_start()

    def end_enroll(self):
        self.enroll_end()

    def get_current_user_data(self):
        fpList = self.get_current_user_info_list()
        return  fpList

    def current_user_fingerprint_counts(self):
        return len(self.get_current_user_data())

    def remove(self, id):
        print 'delete id %d' % id
        return  self.delete_id(id)

    def rename_by_id(self, id, new_name):
        self.rename(id, new_name)

    def close_device(self):
        pass

    def get_device_count(self):
        self.device_count = self.iface.getDevCount()

class RemoveDialog(Gtk.Dialog):
    def __init__(self, parent, fingername):
        Gtk.Dialog.__init__(self, _("Delete"), parent, Gtk.DialogFlags.MODAL,
                            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL))

        self.set_name("remove_dialog")
        self.set_size_request(375, 146)
        self.set_resizable(False)

        label = Gtk.Label()
        label.set_name("remove_dialog_label")

        remove_button = Gtk.Button(_("Delete"))
        remove_button.set_name("remove_button")
        self.add_action_widget(remove_button, Gtk.ResponseType.OK)
        # self.add_button(_("Delete"), Gtk.ResponseType.OK)

        # test
        fpDev.get_device_count()
        if fpDev.device_count > 0:
            text = _("Are you sure you want to delete '%s' ?") % (fingername)
            remove_button.set_sensitive(True)
        else:
            text = _("Unconnected device, can not delete!")
            remove_button.set_sensitive(False)
        label.set_text(text)

        box = self.get_content_area()
        box.set_name("remove_dialog_box")
        # box.set_size_request(375, 112)
        box.add(label)
        self.show_all()

class FailurePromptPage(Gtk.Box):
    def __init__(self, text, button_func):
        super(FailurePromptPage, self).__init__()
        self.set_orientation(Gtk.Orientation.VERTICAL)

        # image
        image = Gtk.Image()
        pixbuf = GdkPixbuf.Pixbuf.new_from_file(imagename)
        image.set_from_pixbuf(pixbuf)
        image_box = Gtk.Box()
        image_box.set_name("prompt_image_box")
        image_box.set_center_widget(image)

        # prompt label
        prompt_label = Gtk.Label()
        prompt_label.set_line_wrap(True)
        prompt_label.set_size_request(300, -1)
        if text == "":
            self.text = _("The device is not recognized, please make sure the sensor is clean or "
                          "offset the finger left and right.")
        else:
            self.text = text
        prompt_label.set_text(self.text)
        prompt_label.set_use_markup(True)
        prompt_label.set_markup('<span foreground="#333333" font="SourceHanSansCN-Normal 16px">%s</span>' % prompt_label.get_text())
        # use GtkTable to wrap line of GtkLabel without making the dialog ridiculously wide.
        table = Gtk.Table(1, 1, False)
        table.set_size_request(300, -1)
        table.attach(prompt_label, 0, 1, 0, 1, Gtk.AttachOptions.SHRINK | Gtk.AttachOptions.FILL)
        table_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        table_box.set_name("table_box")
        table_box.set_center_widget(table)

        # cancel button
        self.cancel_button = Gtk.Button(_("Cancel"))
        self.cancel_button.set_name("button")
        self.cancel_button.set_size_request(100, 30)
        self.cancel_button.connect("clicked", button_func)
        button_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        button_box.set_name("failed_button_box")
        button_box.set_size_request(420, -1)
        button_box.pack_end(self.cancel_button, False, False, 0)

        self.pack_start(image_box, False, False, 0)
        self.pack_start(table_box, False, False, 0)
        self.pack_start(button_box, False, False, 0)

class AddFingerprintDialog(Gtk.Dialog):
    def __init__(self, parent):
        Gtk.Dialog.__init__(self, _("Add Fingerprint"), parent, Gtk.DialogFlags.MODAL)

        self.set_name("add_fingerprint_dialog")
        # self.set_default_size(445, 320)
        self.set_size_request(445, 320)
        self.set_resizable(False)

        self.content_box = self.get_content_area()
        self.content_box.set_name("add_fingerprint_dialog_box")

        self.show_all()

        self.times = 0
        self.source_id = 0

        self.start_page = self.create_start_page()
        self.swipe_prompt_page = self.create_swipe_prompt_page()
        # self.keep_swipe_prompt_page = self.create_keep_swipe_prompt_page()
        # self.failure_prompt_page = self.create_failure_prompt_page()
        self.existence_prompt_page = self.create_existence_prompt_page()
        self.success_prompt_page = self.create_success_prompt_page()

        self.connect("response", self.on_response)

    def remove_all(self):
        widgets = self.content_box.get_children()
        for widget in widgets:
            self.content_box.remove(widget)

    def create_start_page(self):
        self.start_button = Gtk.Button(_("Click to start recording"))
        self.start_button.set_name("button")
        self.start_button.set_size_request(140, 40)
        self.start_button.connect("clicked", self.start_button_clicked)
        start_button_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        start_button_box.set_size_request(140, 40)
        start_button_box.set_center_widget(self.start_button)

        self.user_max_label = Gtk.Label(_("The current user's fingerprint number reaches the maximum"))
        self.user_max_label.set_use_markup(True)
        self.user_max_label.set_markup(
            '<span foreground="#888888" font="SourceHanSansCN-Normal 14px">%s</span>' % self.user_max_label.get_text())
        self.user_max_label_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        self.user_max_label_box.set_name("user_max_label_box")
        self.user_max_label_box.set_center_widget(self.user_max_label)

        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        vbox.pack_start(start_button_box, False, False, 0)
        vbox.pack_start(self.user_max_label_box, False, False, 20)

        box = Gtk.Box()
        # box.props.halign = Gtk.Align.CENTER
        # box.pack_start(self.start_button, False, False, 0)
        box.set_center_widget(vbox)

        return box

    def create_swipe_prompt_page(self):
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        # box.props.halign = Gtk.Align.CENTER

        # image
        image = Gtk.Image()
        pixbuf = GdkPixbuf.Pixbuf.new_from_file(imagename)
        image.set_from_pixbuf(pixbuf)
        image_box = Gtk.Box()
        image_box.set_name("prompt_image_box")
        image_box.set_center_widget(image)

        # prompt label
        prompt_label = Gtk.Label(_("Let your finger swipe on the sensor"))
        prompt_label.set_use_markup(True)
        prompt_label.set_markup('<span foreground="#333333" font="SourceHanSansCN-Normal 16px">%s</span>' % prompt_label.get_text())
        prompt_label_box = Gtk.Box()
        prompt_label_box.set_name("prompt_label_box")
        prompt_label_box.set_center_widget(prompt_label)

        # times prompt label
        times_prompt_label = Gtk.Label(_("Need to record three times, current is the first time"))
        times_prompt_label.set_use_markup(True)
        times_prompt_label.set_markup(
            '<span foreground="#888888" font="SourceHanSansCN-Normal 12px">%s</span>' % times_prompt_label.get_text())
        times_prompt_label_box = Gtk.Box()
        times_prompt_label_box.set_center_widget(times_prompt_label)

        # cancel button
        button = Gtk.Button(_("Cancel"))
        button.set_name("button")
        button.set_size_request(100, 30)
        button.connect("clicked", self.cancel_button_clicked)
        button_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        button_box.set_name("button_box")
        button_box.set_size_request(420, -1)
        button_box.pack_end(button, False, False, 0)

        box.pack_start(image_box, False, False, 0)
        box.pack_start(prompt_label_box, False, False, 0)
        box.pack_start(times_prompt_label_box, False, False, 0)
        box.pack_start(button_box, False, False, 0)

        self.times = 1

        return box

    def create_keep_swipe_prompt_page(self, times):
        print "in ", sys._getframe().f_code.co_name  # debug
        # print "times= ", times
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        # box.props.halign = Gtk.Align.CENTER

        # image
        image = Gtk.Image()
        pixbuf = GdkPixbuf.Pixbuf.new_from_file(imagename)
        image.set_from_pixbuf(pixbuf)
        image_box = Gtk.Box()
        image_box.set_name("prompt_image_box")
        image_box.set_center_widget(image)

        # prompt label
        prompt_label = Gtk.Label(_("Keep swiping until it prompts record is finished"))
        prompt_label.set_use_markup(True)
        prompt_label.set_markup('<span foreground="#333333" font="SourceHanSansCN-Normal 16px">%s</span>' % prompt_label.get_text())
        prompt_label_box = Gtk.Box()
        prompt_label_box.set_name("prompt_label_box")
        prompt_label_box.set_center_widget(prompt_label)

        # times prompt label
        if times == 1:
            times_str = _("first")
        elif times == 2:
            times_str = _("second")
        elif times == 3:
            times_str = _("third")
        else:
            times_str = ""
        times_prompt_label = Gtk.Label(_("The %s time to record") % times_str)
        times_prompt_label.set_use_markup(True)
        times_prompt_label.set_markup(
            '<span foreground="#888888" font="SourceHanSansCN-Normal 12px">%s</span>' % times_prompt_label.get_text())
        times_prompt_label_box = Gtk.Box()
        times_prompt_label_box.set_center_widget(times_prompt_label)

        # cancel button
        button = Gtk.Button(_("Cancel"))
        button.set_name("button")
        button.set_size_request(100, 30)
        button.connect("clicked", self.cancel_button_clicked)
        button_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        button_box.set_name("button_box")
        button_box.set_size_request(420, -1)
        button_box.pack_end(button, False, False, 0)

        box.pack_start(image_box, False, False, 0)
        box.pack_start(prompt_label_box, False, False, 0)
        box.pack_start(times_prompt_label_box, False, False, 0)
        box.pack_start(button_box, False, False, 0)

        return box

    def create_existence_prompt_page(self):
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)

        # image
        image = Gtk.Image()
        pixbuf = GdkPixbuf.Pixbuf.new_from_file(imagename)
        image.set_from_pixbuf(pixbuf)
        image_box = Gtk.Box()
        image_box.set_name("prompt_image_box")
        image_box.set_center_widget(image)

        # prompt label
        prompt_label = Gtk.Label()
        prompt_label.set_line_wrap(True)
        prompt_label.set_size_request(300, -1)
        text = _("The fingerprint of current finger already exists, please change to another finger.")
        prompt_label.set_text(text)
        prompt_label.set_use_markup(True)
        prompt_label.set_markup('<span foreground="#333333" font="SourceHanSansCN-Normal 16px">%s</span>' % prompt_label.get_text())
        # use GtkTable to wrap line of GtkLabel without making the dialog ridiculously wide.
        table = Gtk.Table(1, 1, False)
        table.set_size_request(300, -1)
        table.attach(prompt_label, 0, 1, 0, 1, Gtk.AttachOptions.SHRINK | Gtk.AttachOptions.FILL)
        table_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        table_box.set_name("table_box")
        table_box.set_center_widget(table)

        # cancel button
        button = Gtk.Button(_("OK"))
        button.set_name("button")
        button.set_size_request(100, 30)
        button.connect("clicked", self.ok_button_clicked)
        button_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        button_box.set_name("failed_button_box")
        button_box.set_size_request(420, -1)
        button_box.pack_end(button, False, False, 0)

        box.pack_start(image_box, False, False, 0)
        box.pack_start(table_box, False, False, 0)
        # box.pack_start(times_prompt_label_box, False, False, 0)
        box.pack_start(button_box, False, False, 0)

        return box

    def create_success_prompt_page(self):
        box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)

        success_label = Gtk.Label(_("Recording success!"))
        success_label.set_use_markup(True)
        success_label.set_markup(
            '<span foreground="#333333" font="SourceHanSansCN-Normal 18px">%s</span>' % success_label.get_text())
        success_label_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        success_label_box.set_name("success_label_box")
        success_label_box.set_center_widget(success_label)

        can_use_label = Gtk.Label(_("You can use your fingerprint to unlock device next time."))
        can_use_label.set_use_markup(True)
        can_use_label.set_markup(
            '<span foreground="#333333" font="SourceHanSansCN-Normal 14px">%s</span>' % can_use_label.get_text())
        can_use_label_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        can_use_label_box.set_center_widget(can_use_label)

        keep_add_button = Gtk.LinkButton.new_with_label(uri="", label=_("Keep adding other fingerprints"))
        keep_add_button.set_visited(False)
        keep_add_button.set_has_tooltip(False)
        keep_add_button.connect("activate-link", self.go_to_start_page)
        keep_add_button_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        keep_add_button_box.set_name("keep_add_button_box")
        keep_add_button_box.set_center_widget(keep_add_button)

        # close button
        button = Gtk.Button(_("Close"))
        button.set_name("button")
        button.set_size_request(100, 30)
        button.connect("clicked", self.close_button_clicked)
        close_button_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        close_button_box.set_name("button_box")
        close_button_box.set_size_request(420, -1)
        close_button_box.pack_end(button, False, False, 0)

        box.pack_start(success_label_box, False, False, 0)
        box.pack_start(can_use_label_box, False, False, 0)
        box.pack_start(keep_add_button_box, False, False, 0)
        box.pack_start(close_button_box, False, False, 0)

        return box

    def go_to_page(self, page, pagename):
        print "in ", sys._getframe().f_code.co_name, ": ", pagename  # debug
        self.remove_all()
        self.content_box.props.halign = Gtk.Align.CENTER
        if pagename == "start page":
            self.content_box.props.valign = Gtk.Align.CENTER
        elif pagename == "swipe prompt page":
            self.content_box.props.valign = Gtk.Align.FILL
        elif pagename == "keep swipe prompt page":
            self.content_box.props.valign = Gtk.Align.FILL
        elif pagename == "failure prompt page":
            self.content_box.props.valign = Gtk.Align.FILL
        elif pagename == "existence prompt page":
            self.content_box.props.valign = Gtk.Align.FILL
        elif pagename == "success prompt page":
            self.content_box.props.valign = Gtk.Align.FILL

        self.content_box.pack_start(page, False, False, 0)
        self.content_box.show_all()

    def start_button_clicked(self, button):
        print "in ", sys._getframe().f_code.co_name  # debug
        # test
        # self.go_to_page(self.swipe_prompt_page, "swipe prompt page")
        # self.go_to_page(self.keep_swipe_prompt_page, "keep swipe prompt page")
        # self.failure_prompt_page = FailurePromptPage("", self.cancel_button_clicked)
        # self.go_to_page(self.failure_prompt_page, "failure prompt page")
        # self.start_timer(5, self.go_to_record_page, None)
        # self.go_to_page(self.existence_prompt_page, "existence prompt page")
        # self.start_timer(5, self.go_to_record_page, None)
        # self.go_to_page(self.success_prompt_page, "success prompt page")

        fpDev.connect_signal(self)

        ret = fpDev.start_enroll()
        print "ret= ", ret
        # parameter error
        # if ret == -900:
        # if ret == Fp.GEN_PAR_ERR:
        #     print "Parameter to function error!"
        #     return
        # elif ret == -103:
        # elif ret == Fp.DEV_NOT_OPEN:
        #     # device not open
        #     print "Device not open or exist!"
        #     text = _("The device is not open or does not exist")
        #     self.failure_prompt_page = FailurePromptPage(text, self.cancel_button_clicked)
        #     self.go_to_page(self.failure_prompt_page, "failure prompt page")
        #     return

        self.go_to_page(self.swipe_prompt_page, "swipe prompt page")

    def go_to_start_page(self, link_button):
        self.go_to_page(self.start_page, "start page")
        if fpDev.current_user_fingerprint_counts() < fpDev.get_max_fingerprint():
            self.start_button.set_sensitive(True)
            self.user_max_label_box.hide()
        else:
            self.start_button.set_sensitive(False)
            self.user_max_label_box.show()

        return True

    def cancel_button_clicked(self, button):
        self.response(Gtk.ResponseType.CANCEL)

    def close_button_clicked(self, button):
        self.response(Gtk.ResponseType.CLOSE)

    def ok_button_clicked(self, button):
        print "in ", sys._getframe().f_code.co_name  # debug
        self.response(Gtk.ResponseType.OK)
        print "emit OK response"

    def start_timer(self, interval, func, *data):
        print "in ", sys._getframe().f_code.co_name  # debug
        self.end_timer()
        self.source_id = GLib.timeout_add_seconds(interval, func, data)

    def end_timer(self):
        print "in ", sys._getframe().f_code.co_name  # debug
        if self.source_id > 0:
            GLib.source_remove(self.source_id)

    def on_enroll_one_image(self, fp_dev, times, image):
        GObject.idle_add(self.update_ui, times)

    def update_ui(self, times):
        print "in ", sys._getframe().f_code.co_name  # debug
        print "times= ", times
        self.times = times
        if self.times == 1:
            self.go_to_page(self.swipe_prompt_page, "swipe prompt page")
        elif self.times == 2:
            self.go_to_keep_swipe_page(self.times)
        elif self.times == 3:
            self.go_to_keep_swipe_page(self.times)

        return GLib.SOURCE_REMOVE

    def on_enroll_finished(self, fp_dev, success):
        print "in ", sys._getframe().f_code.co_name  # debug
        GObject.idle_add(self.update_finished, success)

    def update_finished(self, success):
        print "success= ", success
        if success:
            self.go_to_page(self.success_prompt_page, "success prompt page")
        # else:
        #     text = _("Enroll failed!")
        #     self.failure_prompt_page = FailurePromptPage(text, self.cancel_button_clicked)
        #     self.go_to_page(self.failure_prompt_page, "failure prompt page")

        return GLib.SOURCE_REMOVE

    def go_to_existence_page(self, *user_data):
        print "in ", sys._getframe().f_code.co_name  # debug
        self.go_to_page(self.existence_prompt_page, "existence prompt page")

        return GLib.SOURCE_REMOVE

    def go_to_keep_swipe_page(self, times):
        print "in ", sys._getframe().f_code.co_name  # debug
        self.keep_swipe_prompt_page = self.create_keep_swipe_prompt_page(self.times)
        self.go_to_page(self.keep_swipe_prompt_page, "keep swipe prompt page")

    def go_to_failure_page(self, text):
        self.failure_prompt_page = FailurePromptPage(text, self.cancel_button_clicked)
        self.go_to_page(self.failure_prompt_page, "failure prompt page")

    def on_catched_error(self, fp_dev, error):
        print "in ", sys._getframe().f_code.co_name # debug
        print "error code= ", error.code
        # fingerprint repeat
        if error.code == Fp.FEATURE_REPEAT:
            GObject.idle_add(self.go_to_existence_page, None)
        elif error.code == Fp.FEATURE_UNKNOW:
            text = _("Unknown finger feature!")
            self.go_to_failure_page(text)
        elif error.code == Fp.DEVICE_INIT_ERROR:
            text = _("The device init failed! Please re-plug the device.")
            self.go_to_failure_page(text)
        elif error.code == Fp.TEMPLATE_SAVE_ERROR:
            text = _("Enroll failed!")
            self.go_to_failure_page(text)

    def on_catched_canceled(self, fp_dev):
        print "in ", sys._getframe().f_code.co_name  # debug
        text = _("Device is occupied. Current record is canceled. Please re-add.")
        self.go_to_failure_page(text)

    def on_response(self, dialog, response_id):
        print "in ", sys._getframe().f_code.co_name  # debug
        if response_id == Gtk.ResponseType.OK:
            # print "dialog.times= ", self.times
            self.go_to_keep_swipe_page(self.times)

        if response_id == Gtk.ResponseType.CANCEL or \
           response_id == Gtk.ResponseType.CLOSE or \
           response_id == Gtk.ResponseType.DELETE_EVENT:
            fpDev.end_enroll()
            self.destroy()
            print "dialog destroy done"
            fpDev.disconnect_signal(None)

class Module:
    def __init__(self):
        print "Module __init__"
        # only one process is running
        try:
            self.lock_filename = os.path.realpath(__file__)
            print "lock_filename= ", self.lock_filename
            self.lock = Lock(self.lock_filename)
            self.lock.acquire()
        except Exception, e:
            print "another is running"
            Wnck.Screen.get_default().force_update()
            a = Wnck.Screen.get_default()
            for item in a.get_windows():
                if item.get_name() == _("Fingerprint"):
                    item.activate(0)
                    sys.exit()

        self.window = Gtk.Window(Gtk.WindowType.TOPLEVEL)
        print "window new"
        self.window.set_size_request(560, 520)
        self.window.set_resizable(False)
        self.window.set_title(_("Fingerprint"))
        self.window.set_icon_name("cs-fingerprint")
        self.window.connect("destroy", self.quit)
        self.window.show()

        self.vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        print "vbox new"
        self.vbox.set_name("main_box")
        self.vbox.show()
        self.window.add(self.vbox)

        # title label
        self.title_label = Gtk.Label(_("Fingerprint Management"))
        self.title_label.set_use_markup(True)
        self.title_label.set_markup('<span foreground="#333333" font="22px">%s</span>' % self.title_label.get_text())
        self.title_label_hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        self.title_label_hbox.set_name("title_label_box")
        self.title_label_hbox.pack_start(self.title_label, False, False, 0)

        # scrolled window
        self.scrolled_window = Gtk.ScrolledWindow()
        self.scrolled_window.set_name("scrolled_window")
        self.scrolled_window.set_size_request(488, 340)
        self.scrolled_window.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)

        # tree view
        self.treeview = Gtk.TreeView()
        self.treeview.set_headers_visible(False)
        self.treeview.set_activate_on_single_click(True)
        self.scrolled_window.add(self.treeview)

        # set fingername font
        renderer = Gtk.CellRendererText()
        renderer.set_fixed_size(240, 42)
        renderer.set_property("editable", True)
        font = "SourceHanSansCN-Regular 14px"
        renderer.set_property("font", font)
        renderer.set_property("foreground-set", True)
        renderer.set_property("foreground", "#333333")
        renderer.set_property("ellipsize-set", True)
        renderer.set_property("ellipsize", Pango.EllipsizeMode.END)
        renderer.set_property("xalign", 0.05)
        renderer.connect("edited", self.text_edited)
        renderer.connect("editing-started", self.text_editing_started)

        # fingername column
        # column = Gtk.TreeViewColumn("FingerName")
        # column.pack_start(renderer, expand=False)
        # column.set_attributes(renderer, text=0)
        column = Gtk.TreeViewColumn("Finger Name", renderer, text=0)
        # test
        # column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        # column.set_fixed_width(252)
        self.treeview.append_column(column)

        # username cell renderer
        renderer = Gtk.CellRendererText()
        renderer.set_fixed_size(188, 42)
        renderer.set_property("size-set", True)
        renderer.set_property("size", 0)

        # username column
        column = Gtk.TreeViewColumn("User Name", renderer, text=1)
        self.treeview.append_column(column)

        # Delete cell renderer text
        renderer = Gtk.CellRendererText()
        renderer.set_fixed_size(30, 42)
        renderer.set_property("font", font)
        renderer.set_property("foreground-set", True)
        renderer.set_property("foreground", "#1187d4")
        renderer.set_property("underline-set", True)
        renderer.set_property("underline", Pango.Underline.LOW)

        # remove label column
        self.remove_label_column = Gtk.TreeViewColumn("Remove Label", renderer, text=2)
        # column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
        # column.set_fixed_width(50)
        self.treeview.append_column(self.remove_label_column)

        # ID cell renderer
        renderer = Gtk.CellRendererText()
        renderer.set_fixed_size(30, 42)
        renderer.set_property("size-set", True)
        renderer.set_property("size", 0)

        # ID column
        column = Gtk.TreeViewColumn("ID", renderer, text=3)
        self.treeview.append_column(column)

        self.treeview.connect("row-activated", self.remove_item)

        # list store
        self.model = Gtk.ListStore(str, str, str, int)
        self.treeview.set_model(self.model)

        # add fingerprint button
        self.add_fingerprint_button = Gtk.Button()
        self.add_fingerprint_button.set_name("button")
        self.add_fingerprint_button.set_label(_("Add Fingerprint"))
        self.add_fingerprint_button.connect("clicked", self.add_fingerprint)
        self.add_fingerprint_button_hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        self.add_fingerprint_button_hbox.pack_start(self.add_fingerprint_button, False, False, 0)

        # device not connect label
        self.device_not_connect_label = Gtk.Label(_("The device is not connected!"))
        self.device_not_connect_label.set_use_markup(True)
        self.device_not_connect_label.set_markup(
            '<span foreground="#888888" font="SourceHanSansCN-Normal 16px">%s</span>' % self.device_not_connect_label.get_text())
        self.device_not_connect_label_box = Gtk.Box()
        self.device_not_connect_label_box.set_name("device_not_connect_label_box")
        self.device_not_connect_label_box.set_center_widget(self.device_not_connect_label)

        self.vbox.add(self.title_label_hbox)
        self.vbox.add(self.scrolled_window)
        self.vbox.add(self.add_fingerprint_button_hbox)
        self.vbox.add(self.device_not_connect_label_box)

        self.window.show_all()
        print "window show all done"

        self.set_theme(cssfile)

        global fpDev
        fpDev = FpDev()
        print "fpDev=", fpDev
        fpDev.connect("info-changed", self.on_info_changed)

        self.device_count = 0
        self.timer_id = 0
        self.refresh()
        self.update_status()
        # listen fingerprint device's status
        self.start_timer(1, self.update_status_interval, None)

    def set_theme(self, file_theme):
        screen = Gdk.Screen.get_default()
        provider = Gtk.CssProvider()
        provider.load_from_path(file_theme)
        context = Gtk.StyleContext()
        context.add_provider_for_screen(screen, provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)

    def text_editing_started(self, renderer, editable, path):
        print "in ", sys._getframe().f_code.co_name  # debug
        if isinstance(editable, Gtk.Entry):
            editable.set_name("editable")
            editable.set_max_length(256)

    def text_edited(self, renderer, path, new_text):
        print "in ", sys._getframe().f_code.co_name  # debug
        print "new text length= ", len(new_text)
        if len(new_text) > 0 and len(new_text) <= 256:
            iter = self.model.get_iter_from_string(path)
            if iter:
                id = self.model.get_value(iter, 3)
                print "id=", id
                self.model.set(iter, 0, new_text)
                fpDev.rename_by_id(id, new_text)
        # GObject.idle_add(self.text_edited_in_idle, path, new_text)

    # test
    # def text_edited_in_idle(self, path, new_text):
    #     print "in ", sys._getframe().f_code.co_name  # debug
    #     # text process
    #     if len(new_text) > 0 and len(new_text) <= 255:
    #         iter = self.model.get_iter_from_string(path)
    #         if iter:
    #             id = self.model.get_value(iter, 3)
    #             print "id=", id
    #             self.model.set(iter, 0, new_text)
    #             fpDev.rename_by_id(id, new_text)


    def append_item(self, fingername, remove_label):
        iter = self.model.append()
        if iter:
            self.model.set(iter, 0, fingername, 1, remove_label)

    def add_fingerprint(self, button):
        dialog = AddFingerprintDialog(self.window)
        dialog.go_to_start_page(None)

        response = dialog.run()

    def remove_item(self, tree_view, path, column):
        # print "path= %s" % path.to_string()
        if column is self.remove_label_column:
            iter = self.model.get_iter(path)
            if iter:
                fingername = self.model.get_value(iter, 0)
                id = self.model.get_value(iter, 3)
                print "id= %d" % id
                # print "fingername= %s" % fingername

                dialog, response = self.remove_dialog_run(fingername)
                if response == Gtk.ResponseType.OK:
                    GObject.idle_add(self.remove_item_in_idle, id)
                elif response == Gtk.ResponseType.CANCEL:
                    print "remove cancel"
                    pass

                dialog.destroy()

    def remove_item_in_idle(self, id):
        ret = fpDev.remove(id)
        print "remove result= ", ret

    def load_data(self, data):
        print "in ", sys._getframe().f_code.co_name  # debug
        for info in data:
            if info.username == get_current_login_user():
                item_data = []
                item_data.append(info.alias)
                item_data.append(info.username)
                item_data.append(_("Delete"))
                item_data.append(info.fpId)
                print item_data
                self.model.append(item_data)

    def refresh(self):
        print "in ", sys._getframe().f_code.co_name  # debug
        self.model.clear()
        user_data = fpDev.get_current_user_data()
        print "user_data: ", user_data
        # test
        for data in user_data:
            print "data: user = %s, id = %ld, name = %s" % (data.username, data.fpId, data.alias)
        self.load_data(user_data)

    def remove_dialog_run(self, fingername):
        dialog = RemoveDialog(self.window, fingername)
        response = dialog.run()

        return [dialog, response]

    def quit(self, *args):
        self.lock.release()
        fpDev.end_enroll()
        self.end_timer()
        Gtk.main_quit()

    def refresh_in_idle(self, *user_data):
        print "in ", sys._getframe().f_code.co_name  # debug
        self.refresh()

        return GLib.SOURCE_REMOVE

    def on_info_changed(self, fp_dev):
        GObject.idle_add(self.refresh_in_idle, None)

    # test
    # def update_remove_column_status(self):
    #     print "in ", sys._getframe().f_code.co_name  # debug
    #     remove_column = self.treeview.get_column(2)
    #     cells = remove_column.get_cells()
    #     for renderer in cells:
    #         print "FpDev.device_count= ", FpDev.device_count
    #         if FpDev.device_count > 0:
    #             renderer.set_property("size-set", False)
    #         else:
    #             renderer.set_property("size-set", True)
    #             renderer.set_property("size", 0)
    #
    # def update_device_not_connect_label_status(self):
    #     if FpDev.device_count > 0:
    #         self.device_not_connect_label.hide()
    #     else:
    #         self.device_not_connect_label.show()
    #
    # def update_add_fingerprint_button_status(self):
    #     if FpDev.device_count > 0:
    #         self.add_fingerprint_button.set_sensitive(True)
    #     else:
    #         self.add_fingerprint_button.set_sensitive(False)

    def update_status(self):
        print "in ", sys._getframe().f_code.co_name  # debug
        fpDev.get_device_count()
        # print "fpDev.device_count= ", fpDev.device_count

        fingername_column = self.treeview.get_column(0)
        fingername_cells = fingername_column.get_cells()
        remove_column = self.treeview.get_column(2)
        remove_cells = remove_column.get_cells()

        if fpDev.device_count > 0:
            fingername_cells[0].set_property("editable", True)
            # "Delete" column show text
            remove_cells[0].set_property("size-set", False)
            self.device_not_connect_label.hide()
            self.add_fingerprint_button.set_sensitive(True)
        else:
            fingername_cells[0].set_property("editable", False)
            # "Delete" column dont't show text
            remove_cells[0].set_property("size-set", True)
            remove_cells[0].set_property("size", 0)
            self.device_not_connect_label.show()
            self.add_fingerprint_button.set_sensitive(False)

        iter = self.model.get_iter_first()
        if iter:
            while iter is not None:
                path = self.model.get_path(iter)
                self.model.row_changed(path, iter)
                iter = self.model.iter_next(iter)

    def update_status_interval(self, *user_data):
        # print "in ", sys._getframe().f_code.co_name  # debug
        fpDev.get_device_count()
        if self.device_count != fpDev.device_count:
            self.device_count = fpDev.device_count
            self.update_status()

        return GLib.SOURCE_CONTINUE

    def start_timer(self, interval, func, *data):
        print "in ", sys._getframe().f_code.co_name  # debug
        self.end_timer()
        self.timer_id = GLib.timeout_add_seconds(interval, func, data)
        print "timer_id= ", self.timer_id

    def end_timer(self):
        print "in ", sys._getframe().f_code.co_name  # debug
        if self.timer_id > 0:
            GLib.source_remove(self.timer_id)

# test
def Quit(signum, frame):
    print "in ", sys._getframe().f_code.co_name  # debug
    fpDev.end_enroll()
    # fpDev.close_device()
    sys.exit()
    print "exit"

if __name__ == "__main__":
    module = Module()
    signal.signal(signal.SIGINT, module.quit)
    signal.signal(signal.SIGSEGV, Quit)
    Gtk.main()
