#! /usr/bin/python
import sys
import dbus
try:
    sys.path.append('/usr/lib/cdos-settings/modules')
    sys.path.append('/usr/lib/cdos-settings/bin')

    from gi.repository import Gtk, Gio,Gdk
    import gettext
    import os
    import os.path
    import json
except Exception, detail:
    print detail
    sys.exit(1)

SCHEMA = "org.cdos"
gettext.install("cdos-desktop", "/usr/share/cdos-de/locale")
home = os.path.expanduser("~")

class Client():
    def __init__(self):
        bus=dbus.SystemBus()
        service = bus.get_object('org.freedesktop.UPower', "/org/freedesktop/UPower");
        self.get_device = service.get_dbus_method ('EnumerateDevices', 'org.freedesktop.UPower')

    def run(self):
        try:
            return self.get_device()
        except:
            return "no"


class GroupedManager:
    def __init__(self):
    	self.settings = Gio.Settings.new(SCHEMA)
    	self.grouped_list = self.settings.get_strv('grouped-applets')
        self.settings.connect("changed",self._reload)

    	window = Gtk.Window()
    	window.set_size_request(480, 480)

        window.set_position(Gtk.WindowPosition.CENTER)
    	window.set_title(_("Systray Settings"))
        window.set_default_icon_name("preferences-system")
    	window.connect("destroy", lambda q: Gtk.main_quit())

        self.set_theme('/usr/lib/cdos-settings/theme.css')
        self.body=Gtk.ScrolledWindow()

        self.body.get_style_context().add_class('traygroup_box')
        self.body.set_policy(Gtk.PolicyType.NEVER,Gtk.PolicyType.AUTOMATIC)

        self.listBox=Gtk.ListBox()
        self.listBox.get_style_context().add_class("traygroup_listbox")
        self.listBox.set_property("selection-mode",False)

        self.body.add(self.listBox)
        window.add(self.body)

        self._load()
        window.show_all()
        Gtk.main()


    def _load(self):
        exclude = ['xunli@cdos.org', 'keyboard@cdos.org', 'show-desktop@cdos.org', 'systray@cdos.org','menu@cdos.org','scale@cdos.org','window-list@cdos.org','calendar@cdos.org']
        enabled_applets = self.settings.get_strv('enabled-applets')

        list_len=len(self.grouped_list)
    	for element in enabled_applets:
    	    uuid = element.split(':')[3]
    	    box = element.split(':')[1]
            if uuid in exclude or uuid == "grouped-applets@cdos.org":
                continue
            metadata=self.get_metadata(uuid)
            if metadata=={}:
                continue
            it=Gtk.ListBoxRow()
            item=Gtk.Box()
            it.add(item)
            it.get_style_context().add_class("traygroup_listbox_item")
            it.set_size_request(480,50)

            selectText=Gtk.ComboBoxText()
            selectText.get_style_context().add_class("traygroup_listbox_item_select")
            selectText.set_name(uuid)

            name=metadata['name'] if metadata['name'] else uuid

            itemName= Gtk.Label(_(name))
            itemName.get_style_context().add_class("traygroup_listbox_item_label")
            item.pack_start(itemName,False,False,5)
            item.pack_end(selectText,False,False,0)
            selectText.set_size_request(140,30)

    	    if uuid in self.grouped_list:
                selectText.append("group",_("In Group"))
    	        selectText.append("panel",_("On Panel"))
    	    elif box == "right" and uuid not in exclude:
                selectText.append("panel",_("On Panel"))
                selectText.append("group",_("In Group"))
            selectText.set_active(0)
            devices = Client().run()
            if uuid == "power@cdos.org" and not devices:
                selectText.set_sensitive (0)

            selectText.set_margin_right(10)

            self.listBox.insert(it,-1)
            selectText.connect("changed", self.combo_changed)
    def _reload(self,key,data):
        if key=="enabled-applets":
            self._load()

    def combo_changed(self,data):
        selectedId=data.get_active_id()
        uuid=data.get_name()
        if selectedId=="group":
            self.grouped_list.append(uuid)
        elif selectedId=="panel":
            self.grouped_list.remove(uuid)
        self.settings.set_strv('grouped-applets', list(set(self.grouped_list)))

    def set_theme(self,file_theme):
        screen = Gdk.Screen.get_default()
        provider = Gtk.CssProvider()
        provider.load_from_path(file_theme)
        context = Gtk.StyleContext()
        context.add_provider_for_screen(screen, provider, Gtk.STYLE_PROVIDER_PRIORITY_USER)
        self.context=context

    def get_metadata(self,uuid):
        collection_type="applet"
        metadata={}
        metadata=self.load_metadata_in(('%s/.local/share/cdos/%ss/%s') % (home, collection_type,uuid))
        if metadata=={}:
            metadata=self.load_metadata_in(('/usr/share/cdos-de/%ss/%s') % (collection_type,uuid))
        return metadata
    def load_metadata_in(self, directory):
        metadata={}
        if os.path.exists(directory) and os.path.isdir(directory):
            if os.path.exists("%s/metadata.json" % (directory)):
                json_data=open("%s/metadata.json" % (directory)).read()
                metadata = json.loads(json_data)
        return metadata

if __name__ == "__main__":
	app = GroupedManager()
