#!/usr/bin/env python

from SettingsWidgets import *

import platform
import subprocess
import os
import re
import threading
import shlex
import gi
from gi.repository import GdkPixbuf


def killProcess(process):
    process.kill()

def getProcessOut(command1, command2=None):
    timeout = 2.0  # Timeout for any subprocess before aborting it

    lines = []
    if command2 == None:
        p = subprocess.Popen(command1, stdout=subprocess.PIPE)
    else:
        p1 = subprocess.Popen(command1, stdout=subprocess.PIPE)
        p = subprocess.Popen(shlex.split(command2), stdin=p1.stdout, stdout=subprocess.PIPE)
    timer = threading.Timer(timeout, killProcess, [p])
    timer.start()
    while True:
        line = p.stdout.readline()
        if not line:
            break
        if line != '':
            lines.append(line)
    timer.cancel()
    return lines

def getGraphicsInfos():
    cards = {}
    count = 0
    for card in getProcessOut(("lspci")):
        if not "VGA" in card:
            continue
        cardId = card.split()[0]
        cardName = None
        for line in getProcessOut(("lspci", "-v", "-s", cardId)):
            if line.startswith(cardId):
                cardName = (line.split(":")[2].split("(rev")[0].strip())
  
        if cardName:
            cards[count] = (cardName)
            count += 1
    return cards

'''
def getDiskSize():
    temp=os.popen('udisksctl info -b /dev/sda | grep Size | awk -F\":\" \'{print $2}\'').read()
    sizestr=float(temp.strip())
    sizek = round(sizestr)
    sizeT = sizek / 1024 ** 4 * 1.073741824
    sizeG = sizek / 1024 **3
    if sizeG > 900:
        return sizeT
    else:
        return sizeG
'''

def getDiskSize():
    disksize = 0
    moreThanOnce = 0
    for line in getProcessOut(("df", "-l")):
        if line.startswith("/dev/"):
            moreThanOnce += 1
            disksize += float(line.split()[1])

    if (moreThanOnce > 1):
        return disksize, True
    else:
        return disksize, False


def getProcInfos():
    infos = [
        ("/proc/cpuinfo", [("cpu_name", "model name"), ("cpu_siblings", "siblings"), ("cpu_cores", "cpu cores")]),
        ("/proc/meminfo", [("mem_total", "MemTotal")])
    ]

    result = {}
    for (proc, pairs) in infos:
        for line in getProcessOut(("cat", proc)):
            for (key, start) in pairs:
                if line.startswith(start):
                    result[key] = line.split(':', 1)[1].strip()
                    break
    return result

def createSystemInfos():    
    procInfos = getProcInfos()
    infos = []
    (dname, dversion, dsuffix) = platform.linux_distribution()
    arch = platform.machine().replace("_", "-")    
    (memsize, memunit) = procInfos['mem_total'].split(" ")
    processorName = procInfos['cpu_name'].replace("(R)", u"\u00A9").replace("(TM)", u"\u2122")
    if 'cpu_cores' in procInfos:
        processorName = processorName + " x " + procInfos['cpu_cores']
    
    if os.path.exists("/etc/cdos/info"):
        title = commands.getoutput("awk -F \"=\" '/GRUB_TITLE/ {print $2}' /etc/cdos/info")
        infos.append((_("Operating System"), title))    
    else:
        infos.append((_("Operating System"), dname + " " + dversion +  " '" + dsuffix.title() + "' (" + arch + ")"))    
#    if 'CINNAMON_VERSION' in os.environ:            
#        infos.append((_("CDOS Version"), os.environ['CINNAMON_VERSION']))
    infos.append((_("Linux Kernel"), platform.release()))
    if os.path.exists("/var/log/installer/syslog"):
        t=os.stat("/var/log/installer/syslog")
        infos.append((_('System Setup Time'),time.strftime('%Y-%m-%d  %H:%M',time.localtime(t.st_ctime))))
    infos.append((_("Processor"), processorName))
    if memunit == "kB":
        infos.append((_("Memory"), '%.2f %s' % ((float(memsize)/(1024*1024)), _("GB"))))
    else:
        infos.append((_("Memory"), procInfos['mem_total']))

    diskSize, multipleDisks = getDiskSize()
    if (multipleDisks):
        diskText = _("Hard Drives")
    else:
        diskText = _("Hard Drive")
    infos.append((diskText, '%.1f %s' % ((diskSize / (1000*1000)), _("GB"))))
    
    '''
    diskSize = getDiskSize()
    if diskSize <= 10:
        infos.append((_("Hard Drive"), '%d %s' % ((round(diskSize)),_("TB"))))
    else:
        infos.append((_("Hard Drive"), '%d %s' % ((round(diskSize)),_("GB"))))
    '''

    cards = getGraphicsInfos()
    for card in cards:
        infos.append((_("Graphics Card"), cards[card]))

    return infos
    
def get_logo(file_dir):   
    path = ""   
    for root, dirs, files in os.walk(file_dir):
        for file in files:
            if os.path.splitext(file)[1] == '.svg':  
                path = os.path.join(root, file)
    return path

class Module:
    def __init__(self, content_box):
        keywords = _("system, information, details, graphic, sound, kernel, version")
        sidePage = SidePage(_("System Info"), "cs-details1", keywords, content_box, module=self)
        self.sidePage = sidePage
        self.name = "info"
        self.category = "hardware"
        self.comment = _("Display system information")
        
    def on_module_selected(self):
        if not self.loaded:
            print "Loading Info module"
            infos = createSystemInfos()                        

            vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
            logo_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
            logo_image = Gtk.Image()
            if os.path.exists ("/usr/share/pixmaps/logos"):
                if os.path.exists ("/usr/share/pixmaps/logos/fangde-logo.svg"):
                    pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale("/usr/share/pixmaps/logos/fangde-logo.svg", 300, 60, True)
                    logo_image.set_from_pixbuf(pixbuf)
                else:
                    pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(get_logo("/usr/share/pixmaps/logos"), 300, 60, True)
                    logo_image.set_from_pixbuf(pixbuf)
            logo_box.pack_start(logo_image, True, True, 50)
         
            table = Gtk.Table.new(len(infos), 2, False)
            table.set_row_spacings(8)
            table.set_col_spacings(15)

            vbox.pack_start(logo_box, True, True, 20)
            vbox.pack_start(table, True, True, 0)
            self.sidePage.add_widget(vbox)
                

            row = 0
            for (key, value) in infos:
                labelKey = Gtk.Label.new(key)
                labelKey.set_alignment(1, 0.5)
                labelKey.get_style_context().add_class("dim-label")
                labelValue = Gtk.Label.new(value)
                labelValue.set_alignment(0, 0.5)
                table.attach(labelKey, 0, 1, row, row+1)
                table.attach(labelValue, 1, 2, row, row+1)
                row += 1
                
