#!/usr/bin/env python

import os
import commands
import gi
gi.require_version('GUdev', '1.0')
from gi.repository import Gtk, Gdk, GLib, GUdev
from SettingsWidgets import *

class Module:
    def __init__(self, content_box):
        keywords = _("mouse, touchpad, synaptic, double-click")
        sidePage = SidePage(_("Mouse and Touchpad"), "cs-mouse1", keywords, content_box, 520, module=self)
        self.sidePage = sidePage
        self.comment = _("Control mouse and touchpad settings")
        self.name = "mouse"
        self.category = "hardware"

    def on_module_selected(self):
        if not self.loaded:
            print "Loading Mouse module"

            self.tabs = []
            self.mousebox = Gtk.VBox()
            self.touchbox = Gtk.VBox()

            self.notebook = Gtk.Notebook()

            mouse = Gtk.ScrolledWindow()
            mouse.add_with_viewport(self.mousebox)

            touch = Gtk.ScrolledWindow()
            touch.add_with_viewport(self.touchbox)
            
            self.notebook.append_page(mouse, Gtk.Label.new(_("Mouse")))
            if self.check_is_latop() is False :
                self.notebook.append_page(touch, Gtk.Label.new(_("Touchpad")))
            self.notebook.expand = True

            # Mouse
            self.mouse_settings = Gio.Settings.new("org.gnome.desktop.peripherals.mouse")
            self.touch_settings = Gio.Settings.new("org.gnome.desktop.peripherals.touchpad")
            vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)

            section = Section(_("General"))
            section.add(GSettingsCheckButton(_("Left handed (mouse buttons inverted)"), "org.cdos.settings-daemon.peripherals.mouse", "left-handed", None))
            section.add(GSettingsCheckButton(_("Show position of pointer when the Control key is pressed"), "org.cdos.settings-daemon.peripherals.mouse", "locate-pointer", None))
            middle = GSettingsCheckButton(_("Emulate middle click by clicking both left and right buttons"), "org.cdos.settings-daemon.peripherals.mouse", "middle-button-enabled", None)
            section.add(middle)
            self.mouse_settings.bind("middle-emulation", middle, "active", Gio.SettingsBindFlags.DEFAULT)
            vbox.add(section)
            
            vbox.add(Gtk.Separator.new(Gtk.Orientation.HORIZONTAL))

            section = Section(_("Pointer Size"))  
            widget = Gtk.Label()
            widget.set_markup("<i><small>%s</small></i>" % _("Note: All sizes may not be available on certain icon themes"))            
            section.add(widget)
            widget = GSettingsRange(_("Size:"), _("Smaller"), _("Larger"), 24, 50, False, "int", False, "org.cdos.desktop.interface", "cursor-size", None, adjustment_step = 1.0)
            widget.add_mark(24.0, Gtk.PositionType.TOP, None)
            section.add_expand(widget)            
            vbox.add(section)
            
            vbox.add(Gtk.Separator.new(Gtk.Orientation.HORIZONTAL))         
            
            section = Section(_("Pointer Speed"))  
            speed = GSettingsRange(_("Acceleration:"), _("Slow"), _("Fast"), -10.0, 10.0, False, "double", False, "org.cdos.settings-daemon.peripherals.mouse", "motion-acceleration", None, adjustment_step = 1.0)
            section.add_expand(speed)
            self.mouse_settings.bind("speed", speed.content_widget.get_adjustment(), "value", Gio.SettingsBindFlags.DEFAULT)
            #section.add_expand(GSettingsRange(_("Sensitivity:"), _("Low"), _("High"), 1, 10, False, "int", False, "org.cdos.settings-daemon.peripherals.mouse", "motion-threshold", None, adjustment_step = 1))
            vbox.add(section)

            vbox.add(Gtk.Separator.new(Gtk.Orientation.HORIZONTAL))         

            section = Section(_("Double-Click Timeout"))  
            section.add_expand(GSettingsRange(_("Timeout:"), _("Short"), _("Long"), 100, 1000, False, "int", False, "org.cdos.settings-daemon.peripherals.mouse", "double-click", None, adjustment_step = 1))
            widget = Gtk.Button.new_with_label(_("Double-click test"))
            widget.connect("button-press-event", self.test_button_clicked)
            section.add_expand(widget)
            vbox.add(section)

            #vbox.add(Gtk.Separator.new(Gtk.Orientation.HORIZONTAL))         

            #section = Section(_("Drag and drop"))  
            #section.add(GSettingsSpinButton(_("Cdos drag threshold"), "org.cdos", "dnd-drag-threshold", None, 1, 400, 1, 1, _("Pixels")))
            #section.add(GSettingsSpinButton(_("GTK drag threshold"), "org.cdos.settings-daemon.peripherals.mouse", "drag-threshold", None, 1, 400, 1, 1, _("Pixels")))
            #vbox.add(section)            
    
            self.mousebox.pack_start(vbox, False, False, 2)

            # Touchpad

            vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)

            section = Section(_("General"))  
            touch_method = [["enabled", _("Enabled")], ["disabled", _("Disabled")],["disabled-on-external-mouse", _("Disabled on external mouse")]]
            touch_method_combo = GSettingsComboBox(_("Enable touchpad"), "org.gnome.desktop.peripherals.touchpad", "send-events", None, touch_method)
            section.add(touch_method_combo)
            self.dwt = GBindCheckButton(_("Disable touchpad while typing"), self.touch_settings, "disable-while-typing", "send-events")
            section.add(self.dwt)
            self.mouseclicks = GBindCheckButton(_("Enable mouseclicks with touchpad"), self.touch_settings, "tap-to-click", "send-events")
            section.add(self.mouseclicks)

            vbox.add(section)
            vbox.add(Gtk.Separator.new(Gtk.Orientation.HORIZONTAL))

            section = Section(_("Scrolling"))  
            self.natural = GBindCheckButton(_("Enable natural scrolling"), self.touch_settings, "natural-scroll", "send-events")
            section.add(self.natural)
            self.edge = GBindCheckButton(_("Enable edge scrolling"), self.touch_settings, "edge-scrolling-enabled", "send-events")
            section.add(self.edge)
            self.tfs = GBindCheckButton(_("Enable two finger scrolling"), self.touch_settings, "two-finger-scrolling-enabled", "send-events")
            section.add(self.tfs)

            vbox.add(section)
            
            vbox.add(Gtk.Separator.new(Gtk.Orientation.HORIZONTAL))

            section = Section(_("Pointer Speed"))  
            touch_speed = GSettingsRange(_("Acceleration:"), _("Slow"), _("Fast"), -10.0, 10.0, False, "double", False, "org.cdos.settings-daemon.peripherals.touchpad", "motion-acceleration", "org.cdos.settings-daemon.peripherals.touchpad/touchpad-enabled", adjustment_step = 1.0)
            section.add_expand(touch_speed)
            self.touch_settings.bind("speed", touch_speed.content_widget.get_adjustment(), "value", Gio.SettingsBindFlags.DEFAULT)
            vbox.add(section)
            
            self.touchbox.pack_start(vbox, False, False, 2)

            self.sidePage.add_widget(self.notebook)
            #settings.bind_writable(key, self, "sensitive", False)
            self.has_mouse = True
            subsystems = ["input", None]
            self.gudev_client = GUdev.Client.new(subsystems)
            self.gudev_client.connect("uevent", self.on_gudev_uevent)
            self.on_gudev_uevent(self, None, None)
            self.touch_settings.connect("changed::send-events", self.on_dependency_setting_changed)
            self.on_dependency_setting_changed(self, None)

    def set_sensitive(self, sensitive):
        self.dwt.set_sensitive(sensitive)
        self.mouseclicks.set_sensitive(sensitive)
        self.natural.set_sensitive(sensitive)
        self.edge.set_sensitive(sensitive)
        self.tfs.set_sensitive(sensitive)

    def on_dependency_setting_changed(self, settings, dep_key):
        enabled = self.touch_settings.get_string("send-events")
        if enabled == "enabled":
            self.set_sensitive(True)
        elif enabled == "disabled":
            self.set_sensitive(False)
        elif enabled == "disabled-on-external-mouse":
            self.set_sensitive(not self.has_mouse)

    def has_udev_property(self, gudev_device, prop):
        if gudev_device.has_property(prop):
            return True
        parent = gudev_device.get_parent()
        if parent.has_property(prop):
            return True
        return False

    def has_udev_mouse(self):
        devices = self.gudev_client.query_by_subsystem("input")
        for device in devices:
            if device.has_property("ID_INPUT_MOUSE") and not device.has_property("ID_INPUT_POINTINGSTICK") and device.has_property("DEVNAME"):
                return True
        return False

    def on_gudev_uevent(self, udev_client, action, device):
        enabled = self.touch_settings.get_string("send-events")
        if enabled != "disabled-on-external-mouse":
            return
        if action == "add" and self.has_udev_property(device, "ID_INPUT_MOUSE"):
            self.has_mouse = True
        else:
            self.has_mouse = self.has_udev_mouse()
        self.set_sensitive(not self.has_mouse)

    def test_button_clicked(self, widget, event):
        if event.type == Gdk.EventType._2BUTTON_PRESS:
            widget.set_label(_("Success!"))
            GLib.timeout_add(1000, self.reset_test_button, widget)
        return True

    def reset_test_button(self, widget):
        widget.set_label(_("Double-click test"))
        return False

    def check_is_latop(self):
        (state, res) = commands.getstatusoutput('/usr/sbin/laptop-detect -v')
        if state == 0:
            return False
        return True

